@@ -1,37 +1,31 @@
 /*
- * This file is part of libtrace
  *
- * Copyright (c) 2007,2008,2009,2010 The University of Waikato, Hamilton, 
- * New Zealand.
- *
- * Authors: Daniel Lawson 
- *          Perry Lorier
- *          Shane Alcock 
- *          
+ * Copyright (c) 2007-2016 The University of Waikato, Hamilton, New Zealand.
  * All rights reserved.
  *
- * This code has been developed by the University of Waikato WAND 
+ * This file is part of libtrace.
+ *
+ * This code has been developed by the University of Waikato WAND
  * research group. For further information please see http://www.wand.net.nz/
  *
  * libtrace is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * it under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * libtrace is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with libtrace; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
- * $Id$
  *
  */
-
+#ifndef _GNU_SOURCE
 #define _GNU_SOURCE
+#endif
 
 #include "config.h"
 #include "common.h"
@@ -41,7 +35,6 @@
 #include "wandio.h"
 
 #include <sys/stat.h>
-#include <assert.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <stdio.h>
@@ -265,7 +258,8 @@ static int legacy_read_packet(libtrace_t *libtrace, libtrace_packet_t *packet) {
 			packet->type = TRACE_RT_DATA_LEGACY_ETH;
 			break;
 		default:
-			assert(0);
+			trace_set_err(libtrace, TRACE_ERR_BAD_FORMAT, "Invalid trace format type in legacy_read_packet()");
+			return -1;
 	}
 
 	/* This is going to block until we either get an entire record
@@ -276,7 +270,7 @@ static int legacy_read_packet(libtrace_t *libtrace, libtrace_packet_t *packet) {
 						buffer,
 						(size_t)64)) != 64) {
 			if (numbytes < 0) {
-				trace_set_err(libtrace,errno,"read(%s)",libtrace->uridata);
+				trace_set_err(libtrace,TRACE_ERR_WANDIO_FAILED,"read(%s)",libtrace->uridata);
 			} else if (numbytes > 0) {
 				
 				continue;
@@ -317,7 +311,7 @@ static int legacynzix_read_packet(libtrace_t *libtrace, libtrace_packet_t *packe
 		if ((numbytes = wandio_read(libtrace->io, buffer,
 						(size_t)68)) != 68) {
 			if (numbytes < 0) {
-				trace_set_err(libtrace,errno,"read(%s)",libtrace->uridata);
+				trace_set_err(libtrace,TRACE_ERR_WANDIO_FAILED,"read(%s)",libtrace->uridata);
 			} else if (numbytes > 0)
 				continue;
 			return numbytes;
@@ -399,7 +393,12 @@ static int legacynzix_get_capture_length(const libtrace_packet_t *packet UNUSED)
 
 static int legacypos_get_wire_length(const libtrace_packet_t *packet) {
 	legacy_pos_t *lpos = (legacy_pos_t *)packet->header;
-	assert(ntohl(lpos->wlen)>0);
+
+	if (ntohl(lpos->wlen) <= 0) {
+		trace_set_err(packet->trace, TRACE_ERR_BAD_PACKET, "Packet wire length is invalid (%d) "
+			"in legacypos_get_wire_length()", ntohl(lpos->wlen));
+		return -1;
+	}
 	return ntohl(lpos->wlen);
 }
 
@@ -530,6 +529,7 @@ static struct libtrace_format_t legacyatm = {
 	legacy_prepare_packet,		/* prepare_packet */
 	NULL,				/* fin_packet */
 	NULL,				/* write_packet */
+	NULL,				/* flush_output */
 	legacyatm_get_link_type,	/* get_link_type */
 	NULL,				/* get_direction */
 	NULL,				/* set_direction */
@@ -537,6 +537,7 @@ static struct libtrace_format_t legacyatm = {
 	NULL,				/* get_timeval */
 	NULL,				/* get_timespec */
 	NULL,				/* get_seconds */
+	NULL,                           /* get_meta_section */
 	NULL,				/* seek_erf */
 	NULL,				/* seek_timeval */
 	NULL,				/* seek_seconds */
@@ -547,11 +548,12 @@ static struct libtrace_format_t legacyatm = {
 	NULL,				/* get_received_packets */
 	NULL,				/* get_filtered_packets */
 	NULL,				/* get_dropped_packets */
-	NULL,				/* get_captured_packets */
+	NULL,				/* get_statistics */
 	NULL,				/* get_fd */
 	trace_event_trace,		/* trace_event */
 	legacyatm_help,			/* help */
-	NULL				/* next pointer */
+	NULL,				/* next pointer */
+	NON_PARALLEL(false)
 };
 
 static struct libtrace_format_t legacyeth = {
@@ -573,6 +575,7 @@ static struct libtrace_format_t legacyeth = {
 	legacy_prepare_packet,		/* prepare_packet */
 	NULL,				/* fin_packet */
 	NULL,				/* write_packet */
+	NULL,				/* flush_output */
 	legacyeth_get_link_type,	/* get_link_type */
 	NULL,				/* get_direction */
 	NULL,				/* set_direction */
@@ -580,6 +583,7 @@ static struct libtrace_format_t legacyeth = {
 	NULL,				/* get_timeval */
 	NULL,				/* get_timespec */
 	NULL,				/* get_seconds */
+	NULL,                           /* get_meta_section */
 	NULL,				/* seek_erf */
 	NULL,				/* seek_timeval */
 	NULL,				/* seek_seconds */
@@ -590,11 +594,12 @@ static struct libtrace_format_t legacyeth = {
 	NULL,				/* get_received_packets */
 	NULL,				/* get_filtered_packets */
 	NULL,				/* get_dropped_packets */
-	NULL,				/* get_captured_packets */
+	NULL,				/* get_statistics */
 	NULL,				/* get_fd */
 	trace_event_trace,		/* trace_event */
 	legacyeth_help,			/* help */
-	NULL				/* next pointer */
+	NULL,				/* next pointer */
+	NON_PARALLEL(false)
 };
 
 static struct libtrace_format_t legacypos = {
@@ -616,6 +621,7 @@ static struct libtrace_format_t legacypos = {
 	legacy_prepare_packet,		/* prepare_packet */
 	NULL,				/* fin_packet */
 	NULL,				/* write_packet */
+	NULL,				/* flush_output */
 	legacypos_get_link_type,	/* get_link_type */
 	NULL,				/* get_direction */
 	NULL,				/* set_direction */
@@ -623,6 +629,7 @@ static struct libtrace_format_t legacypos = {
 	NULL,				/* get_timeval */
 	NULL,				/* get_timespec */
 	NULL,				/* get_seconds */
+	NULL,                           /* get_meta_section */
 	NULL,				/* seek_erf */
 	NULL,				/* seek_timeval */
 	NULL,				/* seek_seconds */
@@ -633,11 +640,12 @@ static struct libtrace_format_t legacypos = {
 	NULL,				/* get_received_packets */
 	NULL,				/* get_filtered_packets */
 	NULL,				/* get_dropped_packets */
-	NULL,				/* get_captured_packets */
+	NULL,				/* get_statistics */
 	NULL,				/* get_fd */
 	trace_event_trace,		/* trace_event */
 	legacypos_help,			/* help */
 	NULL,				/* next pointer */
+	NON_PARALLEL(false)
 };
 
 static struct libtrace_format_t legacynzix = {
@@ -659,6 +667,7 @@ static struct libtrace_format_t legacynzix = {
 	legacy_prepare_packet,		/* prepare_packet */
 	NULL,				/* fin_packet */
 	NULL,				/* write_packet */
+	NULL,				/* flush_output */
 	legacynzix_get_link_type,	/* get_link_type */
 	NULL,				/* get_direction */
 	NULL,				/* set_direction */
@@ -666,6 +675,7 @@ static struct libtrace_format_t legacynzix = {
 	legacynzix_get_timeval,		/* get_timeval */
 	NULL,				/* get_timespec */
 	NULL,				/* get_seconds */
+	NULL,                           /* get_meta_section */
 	NULL,				/* seek_erf */
 	NULL,				/* seek_timeval */
 	NULL,				/* seek_seconds */
@@ -676,11 +686,12 @@ static struct libtrace_format_t legacynzix = {
 	NULL,				/* get_received_packets */
 	NULL,				/* get_filtered_packets */
 	NULL,				/* get_dropped_packets */
-	NULL,				/* get_captured_packets */
+	NULL,				/* get_statistics */
 	NULL,				/* get_fd */
 	trace_event_trace,		/* trace_event */
 	legacynzix_help,		/* help */
 	NULL,				/* next pointer */
+	NON_PARALLEL(false)
 };
 	
 void legacy_constructor(void) {
