@@ -1,40 +1,31 @@
 /*
- * This file is part of libtrace
  *
- * Copyright (c) 2007-2012 The University of Waikato, Hamilton, 
- * New Zealand.
- *
- * Authors: Daniel Lawson 
- *          Perry Lorier
- *          Shane Alcock 
- *          
+ * Copyright (c) 2007-2016 The University of Waikato, Hamilton, New Zealand.
  * All rights reserved.
  *
- * This code has been developed by the University of Waikato WAND 
+ * This file is part of libtrace.
+ *
+ * This code has been developed by the University of Waikato WAND
  * research group. For further information please see http://www.wand.net.nz/
  *
  * libtrace is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * it under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * libtrace is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with libtrace; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
- * $Id$
  *
  */
-
 #include "libtrace_int.h"
 #include "libtrace.h"
 #include "protocols.h"
-#include <assert.h>
 #include <stdlib.h>
 #include <stdio.h> // fprintf
 
@@ -48,7 +39,14 @@ DLLEXPORT void *trace_get_ospf_header(libtrace_packet_t *packet,
         if (!remaining)
                 remaining = &dummy_rem;
 
-        assert(version != NULL && "version may not be NULL when calling trace_get_ospf_header!");
+	if (!packet) {
+		fprintf(stderr, "NULL packet passed into trace_get_ospf_version()\n");
+		return NULL;
+	}
+	if (!version) {
+		fprintf(stderr, "NULL version passed into trace_get_ospf_version()\n");
+		return NULL;
+	}
 
         ospf = trace_get_transport(packet, &proto, remaining);
 
@@ -69,12 +67,19 @@ DLLEXPORT void *trace_get_ospf_contents_v2(libtrace_ospf_v2_t *header,
         uint8_t dummy_type;
         char *ptr;
 
-        assert(remaining != NULL && "remaining may not be NULL when calling trace_get_ospf_contents!");
+	if (!remaining) {
+		fprintf(stderr, "Remaining may not be NULL when calling trace_get_ospf_contents()\n");
+		return NULL;
+	}
+	if (!header) {
+		fprintf(stderr, "Header may not be NULL when calling trace_get_ospf_contents()\n");
+		return NULL;
+	}
 
         if (!ospf_type)
                 ospf_type = &dummy_type;
 
-        if (!header || *remaining < sizeof(libtrace_ospf_v2_t)) {
+        if (*remaining < sizeof(libtrace_ospf_v2_t)) {
                 *ospf_type = 0;
                 *remaining = 0;
                 return NULL;
@@ -94,9 +99,16 @@ DLLEXPORT unsigned char *trace_get_first_ospf_link_from_router_lsa_v2(
                 uint32_t *remaining) {
 
         unsigned char *link_ptr = NULL;
-        assert(remaining != NULL && "remaining may not be NULL when calling trace_get_first_link_from_router_lsa_v2!");
+	if (!remaining) {
+		fprintf(stderr, "Remaining may not be NULL when calling trace_get_first_link_from_router_lsa_v2()\n");
+		return NULL;
+	}
+	if (!lsa) {
+		fprintf(stderr, "NULL lsa passed into trace_get_first_link_from_router_lsa_v2()\n");
+		return NULL;
+	}
 
-        if (!lsa || *remaining < sizeof(libtrace_ospf_router_lsa_v2_t)) {
+        if (*remaining < sizeof(libtrace_ospf_router_lsa_v2_t)) {
                 *remaining = 0;
                 return NULL;
         }
@@ -113,9 +125,16 @@ DLLEXPORT unsigned char *trace_get_first_ospf_lsa_from_db_desc_v2(
 
         unsigned char *lsa_ptr = NULL;
 
-        assert(remaining != NULL && "remaining may not be NULL when calling trace_get_first_ospf_v2_lsa!");
+	if (!remaining) {
+		fprintf(stderr, "Remaining may not be NULL when calling trace_get_first_ospf_v2_lsa()\n");
+		return NULL;
+	}
+	if (!db_desc) {
+		fprintf(stderr, "db_desc may not be NULL when calling trace_get_first_ospf_v2_lsa()\n");
+		return NULL;
+	}
 
-        if (!db_desc || *remaining < sizeof(libtrace_ospf_db_desc_v2_t)) {
+        if (*remaining < sizeof(libtrace_ospf_db_desc_v2_t)) {
                 *remaining = 0;
                 return NULL;
         }
@@ -132,9 +151,18 @@ DLLEXPORT unsigned char *trace_get_first_ospf_lsa_from_update_v2(
 
         unsigned char *lsa_ptr = NULL;
 
-        assert(remaining != NULL && "remaining may not be NULL when calling trace_get_first_ospf_v2_lsa!");
+	if (!remaining) {
+		fprintf(stderr, "Remaining may not be NULL when calling "
+			"trace_get_first_ospf_lsa_from_update_v2()\n");
+		return NULL;
+	}
+	if (!ls_update) {
+		fprintf(stderr, "ls_update may not be NULL when calling "
+			"trace_get_first_ospf_lsa_from_update_v2()\n");
+		return NULL;
+	}
 
-        if (!ls_update || *remaining < sizeof(libtrace_ospf_ls_update_t)) {
+        if (*remaining < sizeof(libtrace_ospf_ls_update_t)) {
                 *remaining = 0;
                 return NULL;
         }
@@ -150,7 +178,11 @@ DLLEXPORT uint32_t trace_get_ospf_metric_from_as_external_lsa_v2(
 
         uint32_t metric = 0;
 
-        assert(as_lsa);
+	if (!as_lsa) {
+		fprintf(stderr, "NULL as_lsa passed into trace_get_ospf_metric_from_as_external_lsa_v2()\n");
+		/* Return metric of 0 on error? */
+		return metric;
+	}
 
         metric = as_lsa->metric_a << 16;
         metric |= (as_lsa->metric_b << 8);
@@ -164,7 +196,11 @@ DLLEXPORT uint32_t trace_get_ospf_metric_from_summary_lsa_v2(
 
         uint32_t metric = 0;
 
-        assert(sum_lsa);
+	if (!sum_lsa) {
+		fprintf(stderr, "NULL sum_lsa passed into trace_get_ospf_metric_from_summary_lsa_v2()\n");
+		/* Return metric of 0 on error? */
+		return metric;
+	}
 
         metric = sum_lsa->metric_a << 16;
         metric |= (sum_lsa->metric_b << 8);
