@@ -1,40 +1,35 @@
 /*
- * This file is part of libtrace
  *
- * Copyright (c) 2007,2008,2009,2010 The University of Waikato, Hamilton, 
- * New Zealand.
- *
- * Authors: Daniel Lawson 
- *          Perry Lorier
- *          Shane Alcock 
- *          
+ * Copyright (c) 2007-2016 The University of Waikato, Hamilton, New Zealand.
  * All rights reserved.
  *
- * This code has been developed by the University of Waikato WAND 
+ * This file is part of libtrace.
+ *
+ * This code has been developed by the University of Waikato WAND
  * research group. For further information please see http://www.wand.net.nz/
  *
  * libtrace is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * it under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * libtrace is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with libtrace; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
- * $Id$
  *
  */
-
 #include "libtrace_int.h"
 #include "libtrace.h"
 #include "protocols.h"
-#include <assert.h>
+
+#ifdef HAVE_WANDDER
+#include <libwandder_etsili.h>
+#endif
 
 /* This file contains all the protocol decoding functions for the meta-data
  * headers that may be prepended to captured packets.
@@ -128,23 +123,56 @@ static void *trace_get_payload_from_radiotap (const void *link,
 	return (void*) ((char*)link + rtaplen);
 }
 
+static void *trace_get_payload_from_etsili(const void *link,
+                libtrace_linktype_t *type, uint32_t *remaining) {
+
+#ifdef HAVE_WANDDER
+        wandder_etsispec_t *dec;
+        uint8_t *ccptr;
+
+        /* XXX Bit annoying to be creating and freeing this every time */
+        dec = wandder_create_etsili_decoder();
+        wandder_attach_etsili_buffer(dec, (uint8_t *)link, *remaining, false);
+        ccptr = wandder_etsili_get_cc_contents(dec, remaining, NULL, 0);
+        /* Assuming all CCs are IP for now */
+        *type = TRACE_TYPE_NONE;
+        wandder_free_etsili_decoder(dec);
+        return ccptr;
+
+#else
+	(void)link;
+	(void)type;
+        *remaining = 0;
+        return NULL;
+#endif
+
+}
+
 DLLEXPORT void *trace_get_packet_meta(const libtrace_packet_t *packet, 
 		libtrace_linktype_t *linktype,
 		uint32_t *remaining)
 {
 	uint32_t dummyrem;
 	void *pktbuf = NULL;
-	assert(packet != NULL);
-	assert(linktype != NULL);
-	
-	if (remaining == NULL) 
+	if (!packet) {
+		fprintf(stderr, "NULL packet passed into trace_get_packet_meta()");
+		return NULL;
+	}
+	if (!linktype) {
+		fprintf(stderr, "NULL linkype passed into trace_get_packet_meta()");
+		return NULL;
+	}
+
+	if (remaining == NULL)
 		remaining = &dummyrem;
-	
+
 	pktbuf = trace_get_packet_buffer(packet, linktype, remaining);
 	switch (*linktype) {
 		case TRACE_TYPE_LINUX_SLL:
 		case TRACE_TYPE_80211_RADIO:
 		case TRACE_TYPE_80211_PRISM:
+		case TRACE_TYPE_ERF_META:
+                case TRACE_TYPE_ETSILI:
 			return pktbuf;
 		/* Non metadata packets */
 		case TRACE_TYPE_HDLC_POS:
@@ -162,6 +190,9 @@ DLLEXPORT void *trace_get_packet_meta(const libtrace_packet_t *packet,
 		case TRACE_TYPE_NONDATA:
 		case TRACE_TYPE_OPENBSD_LOOP:
 		case TRACE_TYPE_UNKNOWN:
+		case TRACE_TYPE_PCAPNG_META:
+		case TRACE_TYPE_TZSP:
+		case TRACE_TYPE_CONTENT_INVALID:
 			return NULL;
 	}
 
@@ -173,14 +204,23 @@ DLLEXPORT void *trace_get_payload_from_meta(const void *meta,
 		libtrace_linktype_t *linktype,
 		uint32_t *remaining)
 {
-	void *nexthdr; 
-	uint16_t arphrd;
-	uint16_t next;
-	
-	assert(meta != NULL);
-	assert(linktype != NULL);
-	assert(remaining != NULL);
-	
+	void *nexthdr;
+	uint16_t arphrd = 0;
+	uint16_t next = 0;
+
+	if (!meta) {
+		fprintf(stderr, "NULL meta passed into trace_get_payload_from_meta()");
+		return NULL;
+	}
+	if (!linktype) {
+		fprintf(stderr, "NULL linktype passed into trace_get_payload_from_meta()");
+		return NULL;
+	}
+	if (!remaining) {
+		fprintf(stderr, "NULL remaining passed into trace_get_payload_from_meta()");
+		return NULL;
+	}
+
 	switch(*linktype) {
 		case TRACE_TYPE_LINUX_SLL:
 			nexthdr = trace_get_payload_from_linux_sll(meta,
@@ -205,6 +245,13 @@ DLLEXPORT void *trace_get_payload_from_meta(const void *meta,
 			nexthdr = trace_get_payload_from_pflog(meta,
 					linktype, remaining);
 			return nexthdr;
+                case TRACE_TYPE_ETSILI:
+                        nexthdr = trace_get_payload_from_etsili(meta,
+                                        linktype, remaining);
+                        return nexthdr;
+
+		case TRACE_TYPE_PCAPNG_META:
+		case TRACE_TYPE_TZSP:
 		case TRACE_TYPE_HDLC_POS:
 		case TRACE_TYPE_ETH:
 		case TRACE_TYPE_ATM:
@@ -218,7 +265,9 @@ DLLEXPORT void *trace_get_payload_from_meta(const void *meta,
 		case TRACE_TYPE_METADATA:
 		case TRACE_TYPE_NONDATA:
 		case TRACE_TYPE_OPENBSD_LOOP:
+		case TRACE_TYPE_ERF_META:
 		case TRACE_TYPE_UNKNOWN:
+		case TRACE_TYPE_CONTENT_INVALID:
 			/* In this case, the pointer passed in does not point
 			 * to a metadata header and so we cannot get the
 			 * payload.
